require 'rails_helper'

RSpec.describe Integrator::Devise::PasswordsController, type: :controller do
  before { @request.env['devise.mapping'] = Devise.mappings[:integrator_user] }
  routes { Integrator::Engine.routes }

  describe 'GET /password' do
    def request(_attr = {})
      get :new
    end

    context 'if CMS already has at least one user' do
      before { create :user }

      it_behaves_like 'success response'

      it 'render new template' do
        expect(request).to render_template :new
      end
    end

    context 'if CMS has not user' do
      it 'redirect to signup path' do
        expect(request).to redirect_to signup_path
      end
    end
  end


  describe 'POST /password' do
    context 'if CMS already has at least one user' do
      let!(:user) { create :user, email: 'admin@belweb.pro' }

      context 'if user found' do
        def request
          post :create, params: { integrator_user: { email: 'admin@belweb.pro' } }
        end

        it 'send mail' do
          expect { request }.to change(Integrator::Devise::Mailer.deliveries, :size).by(1)
          last_delivery = Integrator::Devise::Mailer.deliveries.last
          expect(last_delivery.to).to include 'admin@belweb.pro'

          # todo
          # expect(last_delivery.to).to include token
        end

        it 'redirect to login path' do
          expect(request).to redirect_to login_path
        end
      end

      context 'if user not found' do
        def request
          post :create, params: { integrator_user: { email: 'admin2@belweb.pro' } }
        end

        it 'not send mail' do
          expect { request }.to_not change(Integrator::Devise::Mailer.deliveries, :size)
        end

        it 'render new template' do
          expect(request).to render_template :new
        end
      end
    end

    context 'if CMS has not user' do
      let(:request) { post :create, params: { integrator_user: { email: 'admin@belweb.pro' } } }

      it 'redirect to signup path' do
        expect(request).to redirect_to signup_path
      end
    end
  end


  describe 'GET /reset' do
    context 'if CMS already has at least one user' do
      let!(:user) { create :user }

      context 'with valid token' do
        def request
          get :edit, params: { reset_password_token: user.send_reset_password_instructions }
        end

        it_behaves_like 'success response'

        it 'render edit template' do
          expect(request).to render_template :edit
        end
      end

      context 'with no token' do
        def request
          get :edit
        end

        it 'redirect to login path' do
          expect(request).to redirect_to login_path
        end
      end
    end

    context 'if CMS has not user' do
      let(:request) { get :edit }

      it 'redirect to signup path' do
        expect(request).to redirect_to signup_path
      end
    end
  end


  describe 'POST /reset' do
    context 'if CMS already has at least one user' do
      let!(:user) { create :user }

      context 'with valid token' do
        context 'with valid password && confirmation' do
          let(:request) { post :update, params: { integrator_user: { reset_password_token: user.send_reset_password_instructions,
                                                           password: '12345678', password_confirmation: '12345678' } } }

          it 'redirect to dashboard (root path)' do
            expect(request).to redirect_to root_path
          end
        end

        context 'with invalid password && confirmation' do
          let(:request) { post :update, params: { integrator_user: { reset_password_token: user.send_reset_password_instructions,
                                                           password: '12345678', password_confirmation: '87654321' } } }

          it 'render new template' do
            expect(request).to render_template :new
          end
        end
      end

      context 'with invalid token' do
        let(:request) { post :update, params: { integrator_user: { reset_password_token: '12345',
                                                           password: '12345678', password_confirmation: '12345678' } } }

        it 'render new template' do
          expect(request).to render_template :new
        end
      end
    end

    context 'if CMS has not user' do
      let(:request) { get :edit }

      it 'redirect to signup path' do
        expect(request).to redirect_to signup_path
      end
    end
  end
end
